/********************************************************
Workspace: IAR+SPL
MCU      : STM8S003k3 (16MHz HSE)
Project  : 220_110
Version  : 0
Date     : Sept, 2017
*********************************************************/
#include "stm8s.h"
#include "iostm003.c"
#include <stdbool.h>
#include <math.h>
#include "bitset.c"
#include "defines.c"
#include "crc16.c"
#include "periph_init.c"
#include "7seg_routines.c"
#include "kwork.c"
#include "wave_gen_routines.c"
#include "adc_routines.c"


volatile u16 tim0,tim1,tim2;
volatile bool iwdg_t2,iwdg_t1;

volatile bool adc3_ready;
volatile bool adc2_ready;
volatile u16  adc3val;
volatile u16  adc2val;                                  

enum  {start,work,LoHV,setup,ocp,flash_crc_err,eeprom_crc_err,cssr_err} dev_st=start;

extern "C"
{
 INTERRUPT_HANDLER(TIM1_X3, 11) 
 {
  static u8 mst=0;
  u16 tmp_table;
  static u8 i=0;
  static bool rck=0, dir; 

  /*ADC measurement section */
  static u32 adc3_cps=0, adc2_cps=0; //ADC3_Vref, ADC2_VHV
  static u8 adc3_cnt=0, adc2_cnt=0;
  u16 adc_tmp;

  adc_tmp = ADC1->DRL;   
  adc_tmp|= ADC1->DRH<<8;  
  
  if(ADC1->CSR&1)
  {
   if(!adc3_ready)
   {
    adc3_cps+= adc_tmp;

    if(adc3_cnt<7){adc3_cnt++;}
    else
    {
     adc3val = adc3_cps>>3;  
     adc3_cnt = adc3_cps = 0;  
     adc3_ready = 1;  
    }
   }
  }
  else
  {
   if(!adc2_ready)
   {
    adc2_cps+= adc_tmp;
    
    adc2_cnt++;
    if(!adc2_cnt)
    {
      adc2val = adc2_cps>>8;
      adc2_cnt = adc2_cps = 0;
      adc2_ready = 1;
    }
   }
  }
  if(!i)ADC1->CSR=3; else ADC1->CSR=2;
  /*end of ADC measurement section */  
  
 /*ultrasonic bridge control section */

  tmp_table=(u16)((u32)s_wave.table[i]*mst>>8);

  if(!i)
  {
   dir=1;
   i=1;
   
   if(rck) 
   { 
    rck=0;  
    TIM1->CCMR1 = pwm1|TIM1_CCMR_OCxPE;
    TIM1->CCMR2 = force_inactive; 
   } 
   else 
   { 
    rck=1;  
    TIM1->CCMR1 = force_inactive;  
    TIM1->CCMR2 = pwm1|TIM1_CCMR_OCxPE; 
    
    mst = s_wave.level;
   }   
   TIM1->EGR|=TIM1_EGR_COMG; 
  }
  else if(dir && i<s_wave.steps-1)
  {
   TIM1->CCR1H=TIM1->CCR2H=tmp_table>>8;
   TIM1->CCR1L=TIM1->CCR2L=(u8)tmp_table;
   i++;
  }
  else
  {
   dir=0; i--;
   TIM1->CCR1H=TIM1->CCR2H=tmp_table>>8;
   TIM1->CCR1L=TIM1->CCR2L=(u8)tmp_table;
  }
 /*end of ultrasonic bridge control section */

  iwdg_t1=1;
  TIM1->SR1 = 0;
 } 

 INTERRUPT_HANDLER(TIM2_OVF, 13)
 {
  if(tim0)tim0--;
  if(tim1)tim1--;
  if(tim2)tim2--;
  
  if(kbtim)kbtim--;
  
  scan_keys();
  display_data();

  iwdg_t2   = 1;
  TIM2->SR1 = 0;
 } 
 
 INTERRUPT_HANDLER(CLK_CSSR, 2)
 {
  TIM1->BKR &=~TIM1_BKR_MOE;

  while(1);  
 }
}

void devcon();

void main()
{    
 cfg.gpio();
 detect_7seg_polarity(ss_pol);
 if(cfg.clk()==false)
 {
  //           
  //        
  dbuf[3]='E',dbuf[2]='H',dbuf[1]='S',dbuf[0]='E',dpp[3]=1; 
  cfg.t2(2E6,1000);
  while(1);
 }
 //   rom,     .      
 //rom        
 #ifdef debug   
 if(TestFlash()==false);   
 #else    
 if(TestFlash()==false)
 {
  dev_st = flash_crc_err;   
  cfg.t2(eep.osc_frq,1000); 
  goto skip_init;
 }   
 #endif     
 cfg.ee(); 
 cfg.t1(eep.osc_frq,spwm_frq);
 cfg.t2(eep.osc_frq,1000);
 cfg.adc();
 ITC_SetSoftwarePriority(ITC_IRQ_TIM1_OVF, ITC_PRIORITYLEVEL_2); 
 ITC_SetSoftwarePriority(ITC_IRQ_TIM2_OVF, ITC_PRIORITYLEVEL_1);
 ITC_SetSoftwarePriority(ITC_IRQ_CLK, ITC_PRIORITYLEVEL_0); 
#ifndef debug 
 cfg.iwdg();
#endif


 skip_init: 
 asm("rim");
 
 endless
 {
  devcon();

  if(iwdg_t2&&iwdg_t1){iwdg_t2=iwdg_t1=0;IWDG_ReloadCounter();}
 }
}

void devcon()
{
 static bool itr=0, btn_trg;
 static u16 Vac_out;
 static u16 V310_tmp;

 if(btn[0]==rls && btn[1]==rls)btn_trg=0;
 else if(!btn_trg && btn[0]==hold && btn[1]==hold && dev_st<setup)  
 {  
  btn_trg=1; 
  itr=0;
  dev_st=setup;  
 }   

 switch (dev_st)
 {
  case start:
  {
   if(!itr) 
   { 
    itr=1; 
    //    
    TIM1->BKR &= ~TIM1_BKR_MOE;
    //  
    REL = 0;
    //      
    dbuf[3]=' ';dbuf[2]='r';dbuf[1]='u';dbuf[0]='n';
    //  prom
    if(TestEE()==false) {dev_st = eeprom_crc_err; goto exit_start;}
    //     
    //     eeprom
    s_wave.base_frq = eep.base_frq;
    calc_base_wave(s_wave,eep.osc_frq/spwm_frq/2-1); 
    //    
    adc3_ready=adc2_ready=0; 
    tim0=1000;
    tim1=0xffff;
    Vac_out = (u16)eep.Vac_out;
   } 
   
   //   
   if(!tim0)
   {
    tim0 = 500;
    static u16 V310=0;
    
    //    
    u16 tmp      = calc_Vcc(adc3val);    
    //     
    V310_tmp = calc_HV(adc2val,tmp);
    //    
    adc3_ready=adc2_ready=0;
    
    //     
    if(V310 < V310_tmp)
    {
     V310 = V310_tmp;
     convert_data(V310);
     dbuf[3]='H';
     dpp[3]=1;dpp[2]=dpp[1]=dpp[0]=0; 
    }
    //       DC
    else if(V310_tmp>(u32)Vac_out*1414*110/1000/100)
    {
     // 
     REL = 1;
     //  spwm
     s_wave.level = calc_wave_level(V310_tmp, eep.Vac_out);
     //   
     dev_st = work;
    }
    //          
    else if(!tim1) dev_st = LoHV;    
   }
  
   exit_start:
	 if(dev_st!=start){itr=0;}
	}
  break;
  case work:
  {
   if(!itr) 
   { 
    itr=1; 
    dpp[3]=1;dpp[2]=dpp[1]=dpp[0]=0;
    TIM1->BKR |= TIM1_BKR_MOE; 
   } 
   
   //     HV
   if(adc3_ready && adc2_ready)
   {
    //    
    u16 tmp      = calc_Vcc(adc3val);    
    //     
    V310_tmp = calc_HV(adc2val,tmp);
    //   
    static u8 v_cnt=0;
    if(V310_tmp<(u32)Vac_out*1414*90/1000/100)
    {
     if(v_cnt<0xff)v_cnt++; 
     else {v_cnt=0; dev_st=LoHV;}
    }
    else if(v_cnt)v_cnt--;    
     
    //    
    adc3_ready=adc2_ready=0;
    //  spwm
    s_wave.level = calc_wave_level(V310_tmp, eep.Vac_out);    
   }
   //     ,   2 
   if(!tim0)
   {
    tim0 = 2000;
    
    static u8 dsel;
    
    if(!dsel)
    {
     dsel=1;
     convert_data(eep.Vac_out);
     dbuf[3]='U';     
    }
    else if(dsel==1)
    {
     dsel=2;
     convert_data(eep.base_frq);       
     dbuf[3]='F';     
    }    
    else 
    {
     dsel=0;
     convert_data(V310_tmp);       
     dbuf[3]='H';     
    }     
   }
   
   #ifdef sc_control
   //         
   {
    #define inputA PB_IDR_IDR7
    #define inputB PB_IDR_IDR6
    #define out_A  PB_IDR_IDR1   
    #define out_B  PB_IDR_IDR0   

    static u8 sc_cnt=0;
    //         3843  
    //     
    if((!inputA && !out_A)||(!inputB && !out_B))
    {
     if(sc_cnt<2)sc_cnt++; 
     else 
     {
      sc_cnt=0;
      dev_st=ocp;
     }
    }
    else sc_cnt=0;
   }
   #endif

	 if(dev_st!=work){itr=0; TIM1->BKR&=~TIM1_BKR_MOE; REL=0;}
	}
  break;  
  case LoHV:
  {
   if(!itr) 
   { 
    itr=1; 
    dbuf[3]='L';dbuf[2]='o';dbuf[1]='H';dbuf[0]='U';  
    dpp[3]=0;dpp[2]=1;dpp[1]=dpp[0]=0; 
    tim0 = 1000;
   } 

   //   
   if(!tim0)
   {
    tim0 = 1000;
    
    //    
    u16 tmp      = calc_Vcc(adc3val);    
    //     
    V310_tmp = calc_HV(adc2val,tmp);
    //    
    adc3_ready=adc2_ready=0;

    if(V310_tmp>(u32)Vac_out*1414*110/1000/100) dev_st = start;
   }  

	 if(dev_st!=LoHV){itr=0;}
	}
  break;
  case setup:
  {
   static u16 base_frq;
   static bool s_trg;
   
   if(!itr) 
   { 
    itr=1; 
    s_trg=0;
    tim0 = 2000;
    tim1 = 10000;
    base_frq = (u16)eep.base_frq;
    Vac_out  = (u16)eep.Vac_out;
   } 
   
   bool tmp_trg = s_trg;
   if(!btn_trg && btn[1]!=rls && btn[0]!=rls)
   {
    btn_trg=1;
    s_trg^=1;
   }
   if(tmp_trg!=s_trg) {tim0 = 1000;tim1=12000;}
   
   
   if(tim0)
   {
    dpp[3]=0;dpp[2]=0;dpp[1]=1;dpp[0]=0;
    if(!s_trg){dbuf[3]='S';dbuf[2]='E';dbuf[1]='t';dbuf[0]='U';}  
    else      {dbuf[3]='S';dbuf[2]='E';dbuf[1]='t';dbuf[0]='F';}
   }
   else
   {
    dpp[3]=1;dpp[2]=0;dpp[1]=0;dpp[0]=0;
    
    u16 tmp;
    if(!s_trg)
    {
     tmp = Vac_out;
     input(Vac_out, 60, 200, (u8)1, (u8)5, 0xff); 
     if(tmp!=Vac_out)
     {
      tim1=10000;
      convert_data(Vac_out);
      dbuf[3]='U';
     }     
    }
    else
    {
     tmp = base_frq;
     input(base_frq, wave_min_frq, wave_max_frq, (u8)1, (u8)5, 0xff); 
     if(tmp!=base_frq)
     {
      tim1=10000;
      convert_data(base_frq);
      dbuf[3]='F';
     }       
    }
   }
    
   if(!tim1) 
   {
    dev_st = start;

    if(base_frq != eep.base_frq || Vac_out != eep.Vac_out)
    {
     FLASH_Unlock(FLASH_MEMTYPE_DATA);
     if(base_frq != eep.base_frq)eep.base_frq = base_frq; 
     if(Vac_out  != eep.Vac_out) eep.Vac_out  = Vac_out;
     
     eep.checksum = Crc16(0x4000,0x4000+sizeof(initval)-2-1);   
     FLASH_Lock(FLASH_MEMTYPE_DATA);
    }
   }

	 if(dev_st!=setup){itr=0;}
	}
  break; 
  case ocp:
  {
   static u8 ocp_cnt=4;
   if(!itr) 
   { 
    itr=1; 
    dbuf[3]='O';dbuf[2]='C';dbuf[1]='P';dbuf[0]='t';  
    dpp[3]=0;dpp[2]=1;dpp[1]=0;dpp[0]=0;  
    tim0=10000;
    //       
    //  
    if(tim2)
    {
     if(ocp_cnt)ocp_cnt--;
    }
    else ocp_cnt=4;  
   } 
  
   //      ,      
   if(!tim0)
   {
    if(!ocp_cnt) {dbuf[3]='o';dbuf[2]='c';dbuf[1]='l';dbuf[0]='c';}
    else dev_st = start; 
   } 

	 if(dev_st!=ocp){itr=0;tim2=0xffff;}
	}
  break;
  case flash_crc_err:
  {
   dbuf[3]='E';dbuf[2]='F';dbuf[1]='L';dbuf[0]='h';  
   dpp[3]=1;dpp[2]=dpp[1]=dpp[0]=0;       
	}
  break;
  case eeprom_crc_err:
  {
   if(!itr) 
   { 
    itr=1;   
    FLASH_Unlock(FLASH_MEMTYPE_DATA);
    eep.init-=1;
    cfg.ee();    
    dbuf[3]='E';dbuf[2]=' ';dbuf[1]='E';dbuf[0]='E';  
    dpp[3]=1;dpp[2]=dpp[1]=dpp[0]=0;   
   }
	}
  break;  
 }
/*
//  case cssr_err:
//  {
//   if(!itr) 
//   { 
//    itr=1;   
// 
//   }
//
//	}
//  break;  
// } 
*/
}